/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Standard image size definitions
 *
 * Copyright (C) 2013, Sylwester Nawrocki <sylvester.nawrocki@gmail.com>
 */
#ifndef _IMAGE_SIZES_H
#define _IMAGE_SIZES_H

#define CIF_WIDTH	352
#define CIF_HEIGHT	288

#define HD_720_WIDTH	1280
#define HD_720_HEIGHT	720

#define HD_1080_WIDTH	1920
#define HD_1080_HEIGHT	1080

#define QCIF_WIDTH	176
#define QCIF_HEIGHT	144

#define QQCIF_WIDTH	88
#define QQCIF_HEIGHT	72

#define QQVGA_WIDTH	160
#define QQVGA_HEIGHT	120

#define QVGA_WIDTH	320
#define QVGA_HEIGHT	240

#define SVGA_WIDTH	800
#define SVGA_HEIGHT	600

#define SXGA_WIDTH	1280
#define SXGA_HEIGHT	1024

#define VGA_WIDTH	640
#define VGA_HEIGHT	480

#define UXGA_WIDTH	1600
#define UXGA_HEIGHT	1200

#define XGA_WIDTH	1024
#define XGA_HEIGHT	768

#endif /* _IMAGE_SIZES_H */
